@@ -2,8 +2,8 @@
 // Created by Gael Aglin on 2019-12-23.
 //
 
-#ifndef RSBS_RCOVER_H
-#define RSBS_RCOVER_H
+#ifndef RCOVER_H
+#define RCOVER_H
 
 #include <stack>
 #include <vector>
@@ -13,7 +13,6 @@
 #include "globals.h"
 #include "dataManager.h"
 #include <cmath>
-#include <algorithm>
 
 using namespace std;
 
@@ -22,15 +21,15 @@ using namespace std;
 class RCover {
 
 public:
-    stack<bitset<M>>* coverWords;
+    stack<bitset<M>, vector<bitset<M>>>* coverWords;
     int* validWords;
-    stack<int> limit;
+    stack<int, vector<int>> limit;
     int nWords;
     DataManager* dm;
-    Supports sup_class = nullptr;
+    ErrorVals sup_class = nullptr;
     int support = -1;
 
-    RCover(DataManager* dmm, vector<float>* weights = nullptr);
+    RCover(DataManager* dmm);
 
     RCover(RCover&& cover) noexcept ;
 
@@ -42,23 +41,25 @@ public:
 
     virtual void intersect(Attribute attribute, bool positive = true) = 0;
 
-    virtual pair<Supports, Support> temporaryIntersect(Attribute attribute, bool positive = true) = 0;
+    virtual pair<ErrorVals, Support> temporaryIntersect(Attribute attribute, bool positive = true) = 0;
 
     Support temporaryIntersectSup(Attribute attribute, bool positive = true);
 
-    Supports minusMe(bitset<M>* cover1);
+    ErrorVal getDiffErrorVal(bitset<M>* cover1, int* valids, int nvalids, bool cover_is_first = false);
 
-    SupportClass countDif(bitset<M>* cover1);
+    ErrorVals getDiffErrorVals(bitset<M>* cover1, bool cover_is_first = false);
 
-    bitset<M>* getTopBitsetArray() const;
+    bitset<M>* getDiffCover(bitset<M>* cover1, bool cover_is_first = false);
+
+    bitset<M>* getTopCover() const;
 
     Support getSupport();
 
-    virtual Supports getSupportPerClass() = 0;
+    virtual ErrorVals getErrorValPerClass() = 0;
 
-    virtual SupportClass countSupportClass(bitset<M>& coverWord, int wordIndex) = 0;
+    virtual ErrorVal getErrorVal(bitset<M>& coverWord, int wordIndex) = 0;
 
-    virtual Supports getSupportPerClass(bitset<M>** cover, int nValidWords, int* validIndexes) = 0;
+    virtual ErrorVals getErrorValPerClass(bitset<M>* cover, int nValidWords, int* validIndexes) = 0;
 
     void backtrack();
 
@@ -95,15 +96,13 @@ public:
                     wordIndex = container->dm->getNClasses();
                 else if (index == 0){
                     wordIndex = index;
-                    //sup = container->getSupportPerClass().first;
                 }
             }
         }
 
         explicit iterator() : wordIndex(-1), container(nullptr) {}
 
-        int getFirstSetBitPos(u_long n)
-        {
+        unsigned int getFirstSetBitPos(unsigned long n) {
             return log2(n & -n) + 1;
         }
 
@@ -148,7 +147,6 @@ public:
                     throw std::out_of_range("Out of Range Exception!");
                 }
                 else {
-//                    cout << "word ind " << wordIndex << endl;
                     return container->sup_class[wordIndex];
                 }
             }
@@ -156,7 +154,6 @@ public:
         }
 
         self_type operator++() {
-            //cout << "bbbbarrive" << endl;
             if (trans_loop)
                 setNextTransID();
             else
@@ -180,22 +177,42 @@ public:
         bool first;
         bitset<M> word;
         bool trans_loop;
-        //int ntrans = -1;
-        //int alltransInd;
 
     };
 
-    iterator begin(bool trans_loop = false)
-    {
+    iterator begin(bool trans_loop = false) {
         return iterator(this, 0, trans_loop);
     }
 
-    iterator end(bool trans_loop = false)
-    {
+    iterator end(bool trans_loop = false) {
         return iterator(this, -1, trans_loop);
     }
 
 };
 
+// custom specialization of std::hash can be injected in namespace std
+namespace std {
+    template<>
+    struct hash<RCover> {
+        std::size_t operator()(const RCover& array) const noexcept {
+            std::size_t h = array.nWords;
+            for (int i = 0; i < array.nWords; ++i) {
+                h ^= array.coverWords[i].top().to_ulong() + 0x9e3779b9 + 64 * h + h / 4;
+            }
+            return h;
+        }
+    };
 
-#endif //RSBS_RCOVER_H
+    template<>
+    struct equal_to<RCover> {
+        bool operator()(const RCover& lhs, const RCover& rhs) const noexcept {
+            for (int i = 0; i < lhs.nWords; ++i) {
+                if (lhs.coverWords[i].top().to_ulong() != rhs.coverWords[i].top().to_ulong()) return false;
+            }
+            return true;
+        }
+    };
+}
+
+
+#endif //RCOVER_H
