@@ -1,7 +1,7 @@
 from .supervised.classifiers.classifier import DL85Classifier
-from .supervised.classifiers.boosting import DL85Booster, MODEL_LP_RATSCH, MODEL_LP_DEMIRIZ, MODEL_QP_MDBOOST
-from .predictors.predictor import DL85Predictor
+from .supervised.classifiers.boosting import DL85Booster, MODEL_LP_RATSCH, MODEL_LP_DEMIRIZ, MODEL_LP_AGLIN, MODEL_QP_MDBOOST
+from .predictors.predictor import DL85Predictor, Cache_Type, Wipe_Type
 from .unsupervised.clustering import DL85Cluster
 from ._version import __version__
 
-__all__ = ['__version__', 'DL85Predictor', 'DL85Classifier', 'DL85Booster', 'DL85Cluster']
+__all__ = ['__version__', 'DL85Predictor', 'DL85Classifier', 'DL85Booster', 'DL85Cluster']
\ No newline at end of file
